// DisplayPane.java - LHM - 1997-09-08
//
// Display various status information for the elastic collision example.
//

package com.lmconsult.elastic;

import java.awt.*;
import java.awt.event.*;

class DisplayPane extends Panel implements ItemListener, Runnable
{ Checkbox mainCheck;
  Checkbox dirCheck;
  Checkbox stopCheck;
  Label systemText;
  Label timeText;
  Label bounceText;
  Label hitText;
  boolean showStuff;
  Thread myThread;
  BasicField field;
  Panel innerPanel;

  DisplayPane()
  { setSize(250,200);
    setLayout(new GridLayout(0,1,5,5));
    showStuff = false;
    mainCheck = new Checkbox("Display status information panel",false);
    mainCheck.addItemListener(this);
    add(mainCheck);
    dirCheck = new Checkbox("Show direction indicators",false);
    dirCheck.addItemListener(this);
    stopCheck = new Checkbox("Stop motion mode",false);
    stopCheck.addItemListener(this);
    systemText = new Label("");
    add(systemText);
    timeText = new Label("");
    add(timeText);
    bounceText = new Label("");
    add(bounceText);
    hitText = new Label("");
    add(hitText);
    add(dirCheck);
    add(stopCheck);
    systemText.setVisible(false);
    timeText.setVisible(false);
    bounceText.setVisible(false);
    hitText.setVisible(false);
    dirCheck.setVisible(false);
    stopCheck.setVisible(false);
    invalidate();
    start();
  }

  public void itemStateChanged(ItemEvent e)
  { if (mainCheck == e.getItemSelectable())
    { showStuff = mainCheck.getState();
      systemText.setVisible(showStuff);
      timeText.setVisible(showStuff);
      bounceText.setVisible(showStuff);
      hitText.setVisible(showStuff);
      dirCheck.setVisible(showStuff);
      stopCheck.setVisible(showStuff);
      invalidate();
    }
    else if (dirCheck == e.getItemSelectable())
    { field.showDirections(dirCheck.getState());
    }
    else if (stopCheck == e.getItemSelectable())
    { field.stopMotion = stopCheck.getState();
    }
  }

  public void start()
  { if (myThread == null) 
    { myThread = new Thread(this);
      myThread.start();
    }
  }

  public void stop()
  { myThread = null;
  }

  public void run()
  { while (myThread != null)
    { try {Thread.sleep(200);} catch (InterruptedException e){}
      if (showStuff && (field != null))
      { systemText.setText("System time = "+Long.toString(field.systemLast));
        timeText.setText("BasicField time = "+Float.toString(field.fieldLast));
        float l = 0;
        if ((field instanceof CollisionField) && !((CollisionField)field).collisions.isEmpty())
        { Collision c = (Collision) ((CollisionField)field).collisions.elementAt(0);
          l = c.time;
        }         
        bounceText.setText("Next collision = "+Float.toString(l));
        long p = 0;
        long w = 0;
        if (field instanceof CollisionField)
        { p = ((CollisionField)field).hitPuck;
          w = ((CollisionField)field).hitWall;
        }
        hitText.setText("Collisions = "+Long.toString(p)+", Hit wall = "+Long.toString(w));
      }
      repaint();
    }
    myThread = null;
  }

  void enrollField(BasicField f)
  { field = f;
  }

  public Insets getInsets()
  { return new Insets(5,5,5,5);
  }

  public void paint(Graphics g)
  { Dimension d = getSize();
    g.setColor(Color.red);
    g.drawRect(0,0,d.width-1,d.height-1);
    g.setColor(Color.yellow);
    g.fillRect(1,1,d.width-2,d.height-2);
  }

}
