package com.lmconsult.elastic;

import java.awt.*;
import java.awt.event.*;

/**
 * This class is a helper class for class BasicField, and supplies the Mouse listening
 * services for picking up the corner and resizing the field.  The basic resize logic
 * is still in BasicField.  We do this so that the BasicField can still operate in a
 * Java 1.0 environment.
 *
 * @author Copyright (c) 1999 Lewis Morton
 * @version 1.0 1999-06-19
 * @see BasicField
 */

public class FieldResizer implements MouseMotionListener, MouseListener
{
  /**
   * The field to be resized
   */
  BasicField helpField;

  /**
   * Are we in a resizing operation?
   */
  boolean dragging=false;
  
  /**
   * How big the helpField can get.
   */
  Dimension maxSize;

  /**
   * Most recent size of the target field.
   */
  Dimension targetSize;
  
  /**
   * Reference lower right corner of the help field at the
   * start of a drag operation.
   */
  Point fixedRef;
  
  /**
   * This is a tiny panel which will be used as the tab to pickup the corner
   * of the field being resized.  It is mouse-sensitive.
   */
  Panel pickup;
  
  /**
   * How big a pickup tab do we want?
   */
  int psize=5;
  
  FieldResizer(BasicField b, Dimension d)
  { helpField=b;
	maxSize=d;
	pickup=new Panel();
	pickup.addMouseMotionListener(this);
	pickup.addMouseListener(this);
	targetSize=b.getSize();
	pickup.setBackground(Color.black);
	pickup.setBounds(targetSize.width-psize-1,targetSize.height-psize-1,psize,psize);
	b.add(pickup);
	b.setCanResize(maxSize);
  }

	/**
     * Invoked when a mouse button is pressed on a component and then 
     * dragged.  Mouse drag events will continue to be delivered to
     * the component where the first originated until the mouse button is
     * released (regardless of whether the mouse position is within the
     * bounds of the component).
     */
    public void mouseDragged(MouseEvent e)
	{ Point p = new Point(psize+1,psize+1);
      if (!dragging)
	  { dragging=true;
		targetSize=helpField.getSize();
		fixedRef=new Point(targetSize.width,targetSize.height);
	    pickup.setBackground(helpField.getBackground());
		pickup.repaint();
		
	  }
	  else
	  { p = e.getPoint();
	  }
	  int newX=fixedRef.x-psize-1+p.x;
	  int newY=fixedRef.y-psize-1+p.y;
	  targetSize = new Dimension(newX,newY);
	  helpField.resizeTransient(targetSize);
	  targetSize=helpField.getSize();
	  helpField.repaint();
	}

    /**
     * Invoked when the mouse button has been moved on a component
     * (with no buttons down).
     */
    public void mouseMoved(MouseEvent e)
	{
	}

    /**
     * Invoked when the mouse has been clicked on a component.
     */
    public void mouseClicked(MouseEvent e)
	{
	}

    /**
     * Invoked when a mouse button has been pressed on a component.
     */
    public void mousePressed(MouseEvent e)
	{
	}

    /**
     * Invoked when a mouse button has been released on a component.
     */
    public void mouseReleased(MouseEvent e)
	{ if (dragging)
	  { dragging=false;
        pickup.setBounds(targetSize.width-6,targetSize.height-6,5,5);
		pickup.setBackground(Color.black);
        pickup.repaint();
		helpField.resizeFinal(targetSize);
		helpField.repaint();
	  }
	}
	
    /**
     * Invoked when the mouse enters a component.
     */
    public void mouseEntered(MouseEvent e)
	{
	}

    /**
     * Invoked when the mouse exits a component.
     */
    public void mouseExited(MouseEvent e)
	{
	}

}
