package com.lmconsult.elastic;

import java.awt.*;

/**
 * This class, together with PuckMover, implements a 'poor-mans Drag-and-Drop'.
 * You should extend this class for any panel you want to have display an object
 * being dragged over it.  If you override paint(), you must call super.paint()
 * at the end of your painting code.  This is a implemented a bit roughly.
 *
 * @author Copyright (c) 1997 Lewis Morton
 * @version 1.0 1999-06-19
 * @see PuckMover
 */

public class FlyoverPanel extends Panel
{ 
  /**
   * This is the thing which will actually do the painting for this panel.
   */
  PuckMover  paintAgent;
  
  /**
   * This registers the drag-and-drop painting agent with this panel.  To
   * make life easier, we decend the container hierarchy looking for instances
   * of FlyoverPanel and register them recursively, so that you don't have to
   * register everything separately.
   */ 
  public void addMover(PuckMover p)
  { paintAgent=p;
	p.registerFlyover(this);
	descend(this,p);
  }
  
  private void descend(Container c, PuckMover p)
  { //	Component[] clist = new Component[getComponentCount()];
	Component clist[]=c.getComponents();
	for (int i=0;i<clist.length;i++)
	{ if (clist[i] instanceof FlyoverPanel) ((FlyoverPanel) clist[i]).addMover(p);
	  else if (clist[i] instanceof Container) descend((Container) clist[i],p);
	}
  }

//  public void update(Graphics g)
//  { paint(g);
//  }

  /**
   * The field must coordinate the painting services of the enclosed objects.
   */
  public void paint(Graphics g)
  { paintAgent.paintFlyer(this,g);
  }
    
}
