// LabelledScrollbar = LHM - 1997-09-10

package com.lmconsult.elastic;

import java.awt.*;
import java.awt.event.*;

public class LabelledScrollbar extends Panel implements AdjustmentListener, LayoutManager
{ String label;
  Scrollbar scroll;
  AdjustmentListener signal;
  int value;
  int max;
  Font calculatedFont;
  Dimension labelBox;
  Dimension tagBox;
  Point textStart;
  Point labelCorner;
  Point tagCorner;
  Color labelBack = Color.cyan;
  int spacer = 2;
  int width = 50;
  int height = 120;
  int wedge = 20;
  int scrollWidth = 15;
  int pad = 1;
  int labWidth;
  int tagWidth;
  int labHeight;

  public LabelledScrollbar(String label,int orient,int val,int vis,int min,int max)
  { setLayout(this);
    scroll = new Scrollbar(orient,val,vis,min,max);
    this.label = label;
    this.value = val;
    this.max = max;
    setSize(width,height);
    scroll.setBounds((width-scrollWidth)/2,wedge,scrollWidth,height-2*wedge);
    add(scroll);
  }

  public void addAdjustmentListener(AdjustmentListener l)
  { signal = l;
    scroll.addAdjustmentListener(this);
  }

  public void adjustmentValueChanged(AdjustmentEvent e)
  { value = e.getValue();
    scroll.setValue(value);
    repaint();
    if (signal != null) signal.adjustmentValueChanged(e);
  }

  public void setValue(int i)
  { value = i;
    scroll.setValue(value);
    repaint();
  }

  public Insets getInsets()
  { return new Insets(20,5,20,5);
  }

  public void paint(Graphics g)
  { Dimension d = getSize();
    Font f = g.getFont();
    if (f != calculatedFont)
    { FontMetrics fm = g.getFontMetrics();
      labWidth = fm.stringWidth(label);
      tagWidth = fm.stringWidth(new Integer(max).toString());
      labHeight = fm.getMaxAscent();
      calculatedFont = f;
    }
    labelBox = new Dimension(labWidth+2*pad,labHeight+2*pad);
    labelCorner = new Point(((int) d.width/2)-((int) labWidth/2)-pad,spacer);
    tagBox = new Dimension(tagWidth+2*pad,labHeight+2*pad);
    tagCorner = new Point(((int) d.width/2)-((int) tagWidth/2)-pad,d.height-spacer-labHeight-2*pad-1);
    textStart = new Point(pad,labHeight+pad);
    g.setColor(labelBack);
    g.fillRect(labelCorner.x,labelCorner.y,labelBox.width,labelBox.height);
    g.fillRect(tagCorner.x,tagCorner.y,tagBox.width,tagBox.height);
    g.setColor(Color.red);
    g.drawRect(0,0,d.width-1,d.height-1);
    g.drawRect(labelCorner.x,labelCorner.y,labelBox.width,labelBox.height);
    g.drawRect(tagCorner.x,tagCorner.y,tagBox.width,tagBox.height);
    g.setColor(Color.black);
    g.drawString(label,labelCorner.x+textStart.x,labelCorner.y+textStart.y);  
    g.drawString(new Integer(value).toString(),tagCorner.x+textStart.x,tagCorner.y+textStart.y);
    paintComponents(g);
  }

  public void addLayoutComponent(String s,Component c)
  {
  }

  public void removeLayoutComponent(Component c)
  {
  }

  public Dimension minimumLayoutSize(Container c)
  { return new Dimension(width,height);
  }

  public Dimension preferredLayoutSize(Container c)
  { return new Dimension(width,height);
  }

  public void layoutContainer(Container c)
  { Dimension d = getSize();
    scroll.setBounds((d.width-scrollWidth)/2,wedge,scrollWidth,d.height-2*wedge);
  }

}
