// Puck.java - LHM - 1997-09-08
//
// The Collidable class must be extended for any object which can be dealt
// with in the Collider.
//
// Puck and Wall are two derived classes for the moving and non-moving
// objects currently implemented.
//

package com.lmconsult.elastic;

import java.lang.Cloneable;
import java.awt.*;

public class Puck extends Collidable implements Cloneable
{ float x,y;			// Puck's position
  float dx,dy;			// Puck's velocity
  float mass;			// Puck's mass
  Color color;			// Puck's rendered color
  Color labelBack;		// Background color for label box
  String label;			// Label for rendering
  Dimension labelBox;		// the size of the enclosing box for the label
  Point labelCorner;		// the upper left corner (relative to the Puck's bounding box)
  Point labelStart;		// the relative point for displaying the label 
  Font calculatedFont;		// the font used for calculating the font box
  Font myFont;			// the current font set by the users
  boolean showMotion;		// render with a velocity vector
  boolean zap;			// flag for special one-tick fancy display/sound
  boolean onField;		// the Puck is on a Field
  BasicField field;		// the containing Field (if onField)
  float radius; 		// puck radius

  Puck()
  { radius = 20;
    color = Color.green;
    labelBack = Color.white;
    calculatedFont = null;
    myFont = null;
    showMotion = false;
    x = 0;
    y = 0;
    dx = 0;
    dy = 0;
    label = null;
  }

  void forward(float milliseconds)
  { if (onField)
    { x += dx*(milliseconds/1000f);
      y += dy*(milliseconds/1000f);
    }
  }

  public void setRadius(int r)
  { radius = (float) r;
    calculatedFont = null;
//    alert();
  }

  public int getRadius()
  { return (int) radius;
  }

  public void setMass(int m)
  { mass = (float) m;
//    alert();
  }

  public int getMass()
  { return (int) mass;
  }

  public void setCenter(Point p)
  { // Set to an (integer) point- we keep the internal values as 'floats'
    x = (float) p.x;
    y = (float) p.y;
//    alert();
  }

  public Point getCenter()
  { // Return integer representation of the center
    return new Point((int) x,(int) y);
  }

  public void setProper(float dx, float dy)
  { this.dx = dx;
    this.dy = dy;
//    alert();
  }

  public void boostProper(float dx, float dy)
  { this.dx += dx;
    this.dy += dy;
//    alert();
  }

  public void setColor(Color c)
  { color = c;
  }

  public Color getColor()
  { return color;
  }

  public void setFont(Font f)
  { myFont = f;
  }

  public Font getFont()
  { return myFont;
  }

  public void setLabel(String newLabel)
  { if (newLabel.equals("")) label = null;
    else label = newLabel;
    calculatedFont = null;
  }

  public String getLabel()
  { return label;
  }

  public synchronized void setOnField(BasicField f)
  { if (!onField) field = f;
    onField = true;
//    alert();
  }

  public synchronized void setOffField()
  { field = null;
    onField = false;
//    alert();
  }

  public void showHighlight(boolean v, boolean m)
  { showHighlight = v;
    showMotion = m;
  }

  public void paint(Graphics g)
  { int x1 = Math.round(x-radius);
    int y1 = Math.round(y-radius);
    int d = Math.round(2*radius);
    g.setColor(color);
    g.fillOval(x1,y1,d-1,d-1);
    if (label != null)
    { if (myFont == null) myFont = g.getFont();
      if (myFont != calculatedFont)
      { g.setFont(myFont);
        FontMetrics fm = g.getFontMetrics();
        int labWidth = fm.stringWidth(label);
        int labHeight = fm.getHeight();
        int offsetHeight = fm.getDescent();
        int pad = 3;
        int xCorn = 0;
        labelBox = new Dimension(labWidth+2*pad,labHeight+2*pad);
        labelCorner = new Point(((int) d/2)-((int) labWidth/2)-pad,((int) d/2)-((int) labHeight/2)-pad);
        labelStart = new Point(((int) d/2)-((int) labWidth/2),((int) d/2)+offsetHeight+pad);
        calculatedFont = myFont;
      }
      g.setColor(labelBack);
      g.fillRect(x1+labelCorner.x,y1+labelCorner.y,labelBox.width,labelBox.height);
      g.setColor(Color.black);
      g.drawRect(x1+labelCorner.x,y1+labelCorner.y,labelBox.width,labelBox.height);
      g.setFont(myFont);
      g.drawString(label,x1+labelStart.x,y1+labelStart.y);  
    }
    g.setColor(Color.black);
    g.drawOval(x1,y1,d-1,d-1);
    if (showHighlight & highlight) g.drawOval(x1+1,y1+1,d-3,d-3);
    if (showMotion) g.drawLine(Math.round(x),Math.round(y),Math.round(x+dx),Math.round(y+dy));
  }

  void alert()
  {
  }

  public int getType()
  { return Collidable.PUCK;
  }

  public Object clone()
  { Object o = null;
    try {o = super.clone();} catch (CloneNotSupportedException e) {}
    return o;
  }
 
}
