// PuckMover - LHM - 1997-09-08
//
// Manage the mouse and transfer of Pucks between Fields.
//

package com.lmconsult.elastic;
import java.awt.*;
import java.awt.event.*;
import java.util.Vector;

class PuckMover implements MouseMotionListener, MouseListener
{ int maxCount;			// The max number of fields which can be registered
  int fieldCount = 0;		// current number of registered fields
  BasicField[] fieldArray;	// the array of registered fields
  boolean puckPicked;		// true if a puck is being dragged
  Puck spare;			// the currently selected Puck
  Puck overPuck;		// the mouse is over this Puck,
  Point overOffset;		// the relative position of the mouse in this Puck
  BasicField overField;		// in this Field
  FlyoverPanel space;		// the parent Container controlled by this PuckMover
  boolean dragging = false;     // the mouse is dragging (ignore clicks and presses!)
  boolean mouseDown = false;	// mouse not down
  boolean crossField = false;	// we have crossed a field boundary while dragging
  Vector flyovers;				// the panels which wish to display dragging objects
  
  PuckMover(FlyoverPanel c, int fCount)
  { fieldArray = new BasicField[fCount];
	flyovers = new Vector(12,12);
    maxCount = fCount;
    fieldCount = 0;
    space = c;
    spare = null;
    space.addMouseListener(this);
    space.addMouseMotionListener(this);
  }

  public void mouseClicked(MouseEvent e)
  {// if (dragging) return;
   // Component comp = e.getComponent();
   // Point compP = getNormLocation(comp);
   // e.translatePoint(compP.x,compP.y);
   // Point p = new Point(e.getPoint());
   // System.out.print("Clicked! x = ");System.out.print(p.x);System.out.print(", y = ");
   // System.out.println(p.y);
   // findPuck(e);
  }

  public void mouseEntered(MouseEvent e)
  {
  } 

  public void mouseExited(MouseEvent e)
  {
  } 

  public void mousePressed(MouseEvent e)
  {
//    System.out.print(e.getID());System.out.print("Pressed: x=");
//    System.out.print(e.getX());System.out.print(": y=");
//    System.out.println(e.getY());
    if (mouseDown & dragging) crossField = true;
    if (mouseDown | dragging) return;
    mouseDown = true;
    crossField = false;
    dragging = false;
    Component comp = e.getComponent();
    Point compP = getNormLocation(comp);
	Point p=e.getPoint();
    p.translate(compP.x,compP.y);
//    Point p = new Point(e.getPoint());
//    System.out.print("Pressed! x = ");System.out.print(p.x);System.out.print(", y = ");
//    System.out.println(p.y);
    findPuck(p);
  }

  public void mouseReleased(MouseEvent e)
  {
//    System.out.print(e.getID());System.out.print("Released: x=");
//    System.out.print(e.getX());System.out.print(": y=");
//    System.out.println(e.getY());
    if (!mouseDown) return;
    Component comp = e.getComponent();
    Point compP = getNormLocation(comp);
    Point p = new Point(e.getPoint());
    p.translate(compP.x,compP.y);
//    System.out.print("Released! x = ");System.out.print(p.x);System.out.print(", y = ");
//    System.out.println(p.y);
//    System.out.print("Dragging = ");System.out.println(dragging);
    BasicField f = findField(p);
    if (f != null)
    { Point pRel = new Point(p);
      Point corn = getNormLocation(f);
      pRel.translate(-corn.x,-corn.y);
      if (puckPicked) pRel.translate(overOffset.x,overOffset.y);
      if (puckPicked && f.couldEnclose(pRel,overPuck.getRadius()+1))
      { if (!dragging)
        { float dx = 0;
          float dy = 0;
          float rad2 = (float) overPuck.getRadius();
          rad2 *= rad2;
          float d2 = overOffset.x*overOffset.x + overOffset.y*overOffset.y;
          float radfrac=(1-(d2/rad2));
//          System.out.print("Boost! frac = ");System.out.println(radfrac);
          if (d2<=1)
          { dx = 10*((float) Math.random())*radfrac;
            dy = 10*((float) Math.random())*radfrac;
          }
          else
          { dx = 10*overOffset.x*radfrac;
            dy = 10*overOffset.y*radfrac;
          }
//          System.out.print("Amount! dx = ");System.out.print(dx);System.out.print(", dy = ");
//          System.out.println(dy);
          overPuck.boostProper(dx,dy);
        }
        else if (!crossField)
        { // throw puck
        }
        if (f.getSpikable())
        { Puck temp=f.removeLastPuck();
          Dimension tdim=f.getSize();
          Point tpnt=new Point(tdim.width/2,tdim.height/2);
          if (temp!=null) tpnt=temp.getCenter();
          overPuck.setCenter(tpnt);
        }
        else overPuck.setCenter(pRel);
        f.addPuck(overPuck);
        f.repaint();
//        System.out.println(f.toString());
        puckPicked = false;
        overPuck = null;
        f.startTime();  // start time if stopped
      }
      else if (!puckPicked)
      { f.stopTime();  // stop/start time;
      }
    }
//    if (puckPicked) f.repaint();
    dragging = false;
    mouseDown = false;
    crossField = false;
  } 

  public void mouseDragged(MouseEvent e)
  {
//    System.out.print(e.getID());System.out.print("Dragged: x=");
//    System.out.print(e.getX());System.out.print(": y=");
//    System.out.println(e.getY());
    if (!mouseDown) return;
    dragging = true;
    Component comp = e.getComponent();
    Point compP = getNormLocation(comp);
    Point p = new Point(e.getPoint());
    p.translate(compP.x,compP.y);
//    System.out.print("Dragged! x = ");System.out.print(p.x);System.out.print(", y = ");
//    System.out.println(p.y);
    if (puckPicked)
    { p.translate(overOffset.x,overOffset.y);
      overPuck.setCenter(p);
//      space.repaint();
//      comp.repaint();
	  for (int i=0;i<flyovers.size();i++)
	  { ((Component) flyovers.elementAt(i)).repaint();
	  }
    }
  }

  public void mouseMoved(MouseEvent e)
  {
  }

  Point getNormLocation(Component c)
  { Component comp = c;
    if (comp == space) return new Point(0,0);
    Rectangle rect = comp.getBounds();
    Point p = new Point(rect.x,rect.y);
    while ((comp.getParent() != null) && (comp.getParent() != space))
    { Container up = comp.getParent();
      Rectangle r = up.getBounds();
      p.translate(r.x,r.y);
      comp = up;
    }
    return p;
  }

  synchronized boolean registerField(BasicField f)
  { if (fieldCount >= maxCount) return false;
    fieldArray[fieldCount++] = f;
    f.addMouseListener(this);
    f.addMouseMotionListener(this);
	return true;
  }
  
  synchronized void registerFlyover(FlyoverPanel f)
  { flyovers.addElement(f);
  }
  
  synchronized BasicField findField(Point p)
  { BasicField f = null;
    for (int i=0;i<fieldCount;i++)
    { f = fieldArray[i];
      Rectangle r = new Rectangle(getNormLocation(f),f.getSize());
      if (r.contains(p)) break;
      f = null;
    }
    return f;
  }

//  boolean findPuck(MouseEvent e)
  boolean findPuck(Point p)
  { boolean got = false;
//    Point p = new Point(e.getPoint());
    BasicField f = findField(p);
    if (f != null)
    { Point pRel = new Point(p);
      Point corn = getNormLocation(f);
      pRel.translate(-corn.x,-corn.y);
      Puck temp = f.extractPuck(pRel);
      if ((temp != null) && f.getClonable())
      { Puck replacer = (Puck) temp.clone();
        f.addPuck(replacer);
//        System.out.println(f.toString());
      }
      if (temp != null)
      { Point center = temp.getCenter();
        center.translate(corn.x,corn.y);
        temp.setCenter(center);
        got = true;
        overPuck = temp;
        overField = f;
        overOffset = new Point(center.x-p.x,center.y-p.y);
      }
      f.repaint();
    }
    else if (puckPicked)
    { got = true;
      overPuck.setCenter(p);
      overOffset = new Point(0,0);
    }
    puckPicked = got;
    return got;
  }

  void paintFlyer(Component c,Graphics g)
  { if (puckPicked)
	{ Point r=getNormLocation(c);
	  Point p=overPuck.getCenter();
	  Point n=new Point(p);
	  n.translate(-r.x,-r.y);
	  overPuck.setCenter(n);
	  overPuck.paint(g);
	  overPuck.setCenter(p);
	}  
//    if (puckPicked)
//    { System.out.print((int) overPuck.x);System.out.print(" ");
//      System.out.println((int) overPuck.y);
//    }
  }
  
  void paint(Graphics g)
  { if (puckPicked)
	{ space.paintComponents(g);
      overPuck.paint(g);
	}
  }  

}
