package com.lmconsult.elastic;

public class Solver
{ double epsilon = 1e-6;

  public double quad (double a, double b, double c)
  { // find smallest positive root of ax^2+bx+c
    double disc = b*b - 4.0*a*c;
    if (disc < epsilon) return Double.POSITIVE_INFINITY;
    double discroot = Math.sqrt(disc);
    double x1 = (-b+discroot)/(2.0*a);
    if (x1<epsilon) return Double.POSITIVE_INFINITY;
    double x2 = (-b-discroot)/(2.0*a);
    return (x2>epsilon) ? x2 : x1;
  }

  public double cube (double a, double b, double c, double d)
  { // find smallest positive root of the cubic ax^3+bx^2+cx+d using Newton's method
    double num, den;
    int i;
 
    double x2 = -10.0;
    double diff = 1.0;
    for (i=0; (diff>epsilon || diff<-epsilon) && i<20; ++i)
    { num = (((a*x2+b)*x2)+c)*x2+d;
      den = (3.0*a*x2+2.0*b)*x2+c;
      if (-0.00001<den && den<0.00001) diff = 0.1+Math.random();
      else diff = num/den;
      x2 -= diff;
    }
    b += a*x2;
    c += b*x2;
    double x3 = quad (a,b,c);
    if (x2>epsilon && x2<x3) x3=x2;
    return x3;
  }

  public double quart (double a, double b, double c, double d, double e)
  { // find smallest positive root of ax^4+bx^3+cx^2+dx+e
    double num, den;
    int i;

    // first find a root using Newton's method
    double x1 = 10.0;
    double diff = 1.0;
    for (i=0; (diff>epsilon || diff<-epsilon) && i<24; ++i)
    { num = ((((a*x1+b)*x1)+c)*x1+d)*x1+e;
      den = ((4.0*a*x1+3.0*b)*x1+2.0*c)*x1+d;
      diff = num/den;
      if (diff<-1e10d || diff>1e10d)
	x1 = 10.0*Math.random();
      else
        x1 -= diff;
    }
    b += a*x1;
    c += b*x1;
    d += c*x1;

    double x2 = cube(a,b,c,d);

    if (x1>epsilon && x1<x2) x2=x1;
    return x2;
  }
}
